#!/bin/bash

# Edit this file to set stuff up.
. /etc/snapshot/settings.text

# idea: run a script before doing the snapshot to mount NFS volumes, mount an LVM static device, etc.
# . /etc/snapshot/before-snapshot.sh

# Function to exit with error.
errexit() {
	echo $@
	exit 1
}

makesymlinks() {
# Function to create .snapshot directories off /home/username.  This is a me-specific thing though.
	temp=`mktemp`
	# This is a function that mirrors the backup root directories for users in a .snapshot directory.
	cd $backuproot;
	for home in /home/*;do 
		[ ! -d $home/.snapshot ] && touch -r $home $temp && mkdir $home/.snapshot && touch -r $temp $home
		for f in *;do 
			[ ! -L $home/.snapshot/$f ] && [ -d $backuproot/$f$home ] && ln -s $backuproot/$f$home $home/.snapshot/$f
			# Can't change the timestamp of symlinks?  gah.
			# [ -L $home/.snapshot/$f ] && touch -r $backuproot/$f$home $home/.snapshot/$f
		done;
	done
	rm $temp
}

type=$1 # hourly, daily, monthly snapshot?
# This simplifies things down below so I can use $max to delete
# the overrotated snapshot, as well as $hourlymax/$dailymax to do
# rotation checks.
[ "$type" == "hourly" ] && max=hourlymax
[ "$type" == "daily" ] && max=dailymax
[ "$type" == "weekly" ] && max=weeklymax
[ "$type" == "monthly" ] && max=monthlymax
[ "$type" == "symlinks" ] && makesymlinks

[ -z "$max" ] && errexit syntax: $0 {hourly, daily,weekly,monthly}


# Rotate the current list of backups, if we can.
if [ -d $backuproot/$type.1 ]; then
	oldest=`ls -dt $backuproot/$type.* | tail -n 1 | sed 's/^.*\.//'`
	for i in `seq $oldest 1`; do
		mv $backuproot/$type.$i $backuproot/$type.$(( i + 1 ))
	done
fi

if [ "$type" == "hourly" ]; then
	# recreate directory structure with hard links.
	cp -al $backuproot/$type.2 $backuproot/$type.1
	rsync -R -va --delete --delete-excluded --exclude-from=$excludefile $backup $backuproot/$type.1/
	touch $backuproot/$type.1
elif [ "$type" == "daily" ]; then
	[ -d $backuproot/hourly.$hourlymax ] && mv $backuproot/hourly.$hourlymax $backuproot/daily.1
	# additional backups to do?
	[ -e /etc/snapshot/daily.sh ] && . /etc/snapshot/daily.sh
elif [  "$type" == "weekly" ]; then
	[ -d $backuproot/daily.$dailymax ] && mv $backuproot/daily.$dailymax $backuproot/weekly.1
	# additional weekly backups to do?
	[ -e /etc/snapshot/weekly.sh ] && . /etc/snapshot/weekly.sh
elif [  "$type" == "monthly" ]; then
	[ -d $backuproot/weekly.$weeklymax ] && mv $backuproot/weekly.$weeklymax $backuproot/monthly.1
	[ -e /etc/snapshot/monthly.sh ] && . /etc/snapshot/monthly.sh
fi

# if we've rotated the last backup off the stack, remove it.
[ -d $backuproot/$type.$((max+1)) ] && rm -rf $backuproot/$type.$((max+1))

# . /etc/snapshot/after-snapshot.sh
